#if UNITY_5_3_OR_NEWER
using UnityEngine;

namespace RoboidControl.Unity {

	/// <summary>
	/// The Unity representation of the TouchSensor
	/// </summary>
	public class TouchSensor : Thing {

		// public SiteServer participant;
		/// <summary>
		/// The core touch sensor
		/// </summary>
		public RoboidControl.TouchSensor coreSensor {
			get => base.core as RoboidControl.TouchSensor;
			set => base.core = value;
		}

		SphereCollider touchCollider = null;

		/// <summary>
		/// Start the Unity represention
		/// </summary>
		protected virtual void Start() {
			if (core == null) {
				participant = FindAnyObjectByType<SiteServer>();
				SetCoreThing(new RoboidControl.TouchSensor(participant.site));
			}
			touchCollider = GetComponent<SphereCollider>();
		}

		/// <summary>
		/// Create the Unity representation
		/// </summary>
		/// <param name="core">The core touch sensor</param>
		/// <returns>The Unity representation of the touch sensor</returns>
		public static TouchSensor Create(RoboidControl.TouchSensor core) {
			GameObject prefab = (GameObject)Resources.Load("TouchSensor");
			if (prefab != null) {
				// Use resource prefab when available
				GameObject gameObj = Instantiate(prefab);
				TouchSensor component = gameObj.GetComponent<TouchSensor>();
				if (component != null)
					component.core = core;
				return component;
			}
			else {
				// Fallback implementation
				GameObject gameObj = new(core.name);
				TouchSensor component = gameObj.AddComponent<TouchSensor>();
				component.Init(core);

				Rigidbody rb = gameObj.AddComponent<Rigidbody>();
				rb.isKinematic = true;

				SphereCollider collider = gameObj.AddComponent<SphereCollider>();
				collider.radius = 0.01f;
				collider.isTrigger = true;
				return component;
			}
		}

		private void OnTriggerEnter(Collider other) {
			// Debug.Log("Touch?");
			if (other.isTrigger) {
				// Debug.Log($"   was trigger {other.name}");
				return;
			}
			if (this.transform.root == other.transform.root) {
				Debug.Log($"   was myself {other.name}");
				return;
			}

			Debug.Log($"*** {this} Touch");
			this.coreSensor.touchedSomething = true;		
			this.core.updateQueue.Enqueue(new RoboidControl.Thing.UpdateEvent(BinaryMsg.Id));
		}
		private void OnTriggerExit(Collider other) {
			if (other.isTrigger)
				return;

			Debug.Log($"*** {this} Touch end");
			this.coreSensor.touchedSomething = false;
		}
	}
}
#endif